unit FlatDSDemo_mf;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,
  // specific units
  wdcDset,WcsFlatFileType,WcsFFDataset,
  // ide added units
  StdCtrls, ExtCtrls, DBCtrls, Grids, DBGrids, DB, ComCtrls, Buttons, Mask;
{DBTABLES}
type
  TForm1 = class(TForm)
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    OD: TOpenDialog;
    SD: TSaveDialog;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    cbLayout: TComboBox;
    Button1: TButton;
    StringGrid1: TStringGrid;
    Memo1: TMemo;
    bLoadDF: TButton;
    tsslds: TTabSheet;
    Panel1: TPanel;
    bOpen: TSpeedButton;
    bClose: TSpeedButton;
    DataSource2: TDataSource;
    LblRecs: TLabel;
    ODDS: TOpenDialog;
    SDDS: TSaveDialog;
    cbQuoted: TCheckBox;
    bSaveDef: TButton;
    CheckBox1: TCheckBox;
    Button6: TButton;
    Splitter1: TSplitter;
    Panel2: TPanel;
    Panel3: TPanel;
    DBMemo1: TDBMemo;
    Panel4: TPanel;
    DBEdit1: TDBEdit;
    DBText1: TDBText;
    bPres: TButton;
    bNext: TButton;
    Panel5: TPanel;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure bOpenClick(Sender: TObject);
    procedure bCloseClick(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure DBGrid1TitleClick(Column: TColumn);
    procedure bLoadDFClick(Sender: TObject);
    procedure bSaveDefClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure cbLayoutClick(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure bNextClick(Sender: TObject);
  private
    { Private declarations }
    cfgfilename: String;
    SLDS: TWcsArticleDataSet;
    FFDS: TWcsFlatFileDataset;
    fSelectedDS: TDataset;
    MSLayout : TMemoryStream;
    procedure LoadCfg;
    procedure SaveCfg;
    procedure SetSelectedDS(const Value: TDataset);
    procedure LoadFormat(Sender: TObject; var Stream: TStream;
      const ForOutput: Boolean);
    procedure LoadDFToGrid(FDL: TFieldDefinitionList);
  public
    { Public declarations }
    property SelectedDS : TDataset read fSelectedDS write SetSelectedDS;
  end;

var
  Form1: TForm1;

implementation
{$R *.dfm}
uses
  inifiles;

procedure TForm1.SaveCfg;
var
  cfg: Tinifile;
  cfgCount, i: integer;
  Layout: String;
begin
// Save list of file definitions
  cfg:= Tinifile.Create(cfgfileName);
  try
    cfgCount:= cfg.readinteger('General', 'LayoutCount', 0);
    for i := 0 to cfgCount-1 do
    begin
      Layout := cfg.ReadString('File Layouts', Format('Layout%2.2d',[i]),'');
      if (Length(Layout) > 0) and FileExists(Layout) then
        cbLayout.items.Add(Layout);
    end;
  finally
    cfg.Free;
  end;
end;

procedure TForm1.LoadCfg;
var
  cfg: Tinifile;
  i: integer;
begin
// Load list of file definitions
  cfgfileName := ExtractFilePath(ParamStr(0))+ 'FlastDSDemo.ini';
  cfg:= Tinifile.Create(cfgfileName);
  try
    cfg.writeinteger('General', 'LayoutCount', cbLayout.items.count);
    cfg.EraseSection('File Layouts');
    for i := 0 to cbLayout.Items.Count-1 do
      cfg.WriteString('File Layouts', Format('Layout%2.2d',[i]),cbLayout.items[i]);
  finally
    cfg.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  MSLayout := TMemoryStream.Create;
  StringGrid1.Cells[1, 0] := 'Index';
  StringGrid1.Cells[2, 0] := 'Name';
  StringGrid1.Cells[3, 0] := 'Type';
  StringGrid1.Cells[4, 0] := 'Size';
  SLDS:= TWcsArticleDataSet.Create(Self);
  FFDS:= TWcsFlatFileDataset.Create(Self);
  DataSource1.DataSet := SLDS;
  DataSource2.DataSet := FFDS;
  LoadCfg;
  PageControl1.ActivePage := tsslds;
  SelectedDS := SLDS
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if MSLayout <> Nil then
  begin
    MSLayout.Free;
    MSLayout := Nil;
    SaveCfg;
  end;
end;


procedure TForm1.bOpenClick(Sender: TObject);
begin
  try
    fSelectedDS.Open;
    LblRecs.Caption := IntToStr(fSelectedDS.RecordCount);
  except
    bClose.Down := true;
    raise;
  end;
end;

procedure TForm1.bCloseClick(Sender: TObject);
begin
  fSelectedDS.Close;
  LblRecs.Caption := '/';
end;

procedure TForm1.PageControl1Change(Sender: TObject);
begin
  if PageControl1.ActivePage = tsslds then
    SelectedDS := SLDS
  else
    SelectedDS := FFDS;
end;

procedure TForm1.SetSelectedDS(const Value: TDataset);
begin
  if fSelectedDS <> Value then
  begin
    if fSelectedDS <> nil then
    begin
      fSelectedDS.Close;
      bClose.Down := true;
    end;
    fSelectedDS := Value;
    if fSelectedDS = SLDS then
    begin
      DBGrid1.DataSource := DataSource1;
      DBNavigator1.DataSource := DataSource1;
    end else begin
      DBGrid1.DataSource := DataSource2;
      DBNavigator1.DataSource := DataSource2;
    end;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  SLDS.AlphaSort;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  SLDS.IDSort;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
  FS : TFileStream;
begin
  if not SDDS.Execute then Exit;
  if SLDS.State in dsEditModes then SLDS.Post;
  FS := TFileStream.Create(SDDS.FileName, fmCreate);
  try
    SLDS.SaveToStream(FS);
  finally
    FS.Free;
  end;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  FS : TFileStream;
begin
  if Sender = bPres then
    ODDS.FileName := ExtractFilePath(ParamStr(0)) + '..\Resources\FlatDatasetDemo.dmo'
  else
    if not ODDS.Execute then Exit;
  FS := TFileStream.Create(ODDS.FileName, fmOpenRead);
  try
    SLDS.LoadFromStream(FS);
  finally
    FS.Free;
  end;
  if Sender = bPres then
  begin
    WindowState := wsMaximized;
    PageControl1.Height := 500;
    bOpenClick(bOpen);
    bOpen.Down := True;
    bNext.Enabled := True;
  end;  
end;

procedure TForm1.LoadFormat(Sender: TObject; var Stream: TStream; const ForOutput: Boolean);
begin
  MSLayout.Position := 0;
  Stream := MSLayout;
end;

procedure TForm1.DBGrid1TitleClick(Column: TColumn);
begin
  Screen.Cursor := crHourGlass;
  Enabled := False;
  try
    FFDS.FFIndexFields := Column.Field.FieldName;
  finally
    Screen.Cursor := crDefault;
    Enabled := True;
  end;
end;

procedure TForm1.LoadDFToGrid(FDL : TFieldDefinitionList);
var
  i : integer;
begin
  StringGrid1.RowCount := FDL.Count;
  for i := 0 to FDL.COunt - 1 do
  with FDL[i] do
  begin
    StringGrid1.Cells[1, i] := IntToStr(i);
    StringGrid1.Cells[2, i] := DName;
    StringGrid1.Cells[3, i] := sWCSFlatFileSrcType[DSrcType];
    StringGrid1.Cells[4, i] := IntToStr(DSrcSize);
  end;
end;

procedure TForm1.Button6Click(Sender: TObject);
var
  TF : TextFile;
  i : integer;
  S : String;
begin
  if not ODDS.Execute then Exit;
  Memo1.Lines.Clear;
  AssignFile(TF,ODDS.FileName);
  Reset(TF);
  for i := 1 to 5 do
    if eof(TF) then
      break
    else begin
      Readln(TF, S);
      Memo1.Lines.Add(S);
    end;
  CloseFile(TF);
  bLoadDF.Enabled := True;
end;

procedure TForm1.bLoadDFClick(Sender: TObject);
var
  FDL : TFieldDefinitionList;
  TF : TextFile;
  i : integer;
  S : String;
begin
  FFDS.Close;
  FFDS.FFIndexFields := '';
  FFDS.FFDataFile := ODDS.FileName;
  FDL := TFieldDefinitionList.Create;
  try
    FDL.Edit;
    FDL.HeaderLines := 1;
    FDL.Family := ffdf_Delimited;
    if cbQuoted.Checked then
      FDL.SetQuotes('"','"','"','"');
    FDL.RecDelim := #13#10;
    FDL.FldDelim := ',';
    FDL.Post;
    LblRecs.Caption := IntToStr( FFDS.PreScanToLayout(FDL));
    LoadDFToGrid(FDL);
    MSLayout.Clear;
    FDL.StreamOut(MSLayout);
    cbLayout.ItemIndex := -1;
  finally
    FDL.Free;
  end;
  FFDS.FFOnGetLayoutStream := LoadFormat;
end;

procedure TForm1.bSaveDefClick(Sender: TObject);
begin
  if not SD.Execute then Exit;
  MSLayout.SaveToFile(SD.FileName);
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i : integer;
begin
  if not OD.Execute then Exit;
  i := cbLayout.Items.Indexof(OD.FileName);
  if i < 0 then
    i := cbLayout.Items.Add(OD.FileName);
  cbLayout.ItemIndex := i;
  cbLayoutClick(cbLayout);
end;

procedure TForm1.cbLayoutClick(Sender: TObject);
var
  FSLayout: TFileStream;
  FDL : TFieldDefinitionList;
begin
  if cbLayout.ItemIndex >= 0 then
  begin
    FSLayout  := TFileStream.Create(cbLayout.Text, fmOpenRead);
    FDL := TFieldDefinitionList.Create;
    try
      FDL.StreamIn(FSLayout);
      LoadDFToGrid(FDL);
    finally
      FDL.Free;
      FSLayout.Free;
    end;


  end;
end;

procedure TForm1.bNextClick(Sender: TObject);
begin
  SLDS.Next;
  if SLDS.Eof then SLDS.First;
end;

end.
